using System;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Demo
{
    /// <summary>
    /// The derived frame class for our tanks
    /// </summary>
    public class TankFrame : Frame
    {
        // Store the combined transformation matrix
        private Matrix combined = Matrix.Identity;
        public Matrix CombinedTransformationMatrix
        {
            get { return combined; } set { combined = value; }
        }
    }
    /// <summary>
    /// The derived mesh container class for our tanks
    /// </summary>
    public class TankMeshContainer : MeshContainer
    {
        // We need no extra data here
    }
    /// <summary>
    /// Will be used to control allocating the hierarchy for a tank
    /// </summary>
    public class TankAllocHierarchy : AllocateHierarchy
    {
        #region Abstract Members
        /// <summary>
        /// Create a new hierarchy frame
        /// </summary>
        public override Frame CreateFrame(string name)
        {
            // Create the new derived tank frame 
            TankFrame frame = new TankFrame();
            // Store the variables
            frame.Name = name; 
            frame.TransformationMatrix = Matrix.Identity;
            frame.CombinedTransformationMatrix = Matrix.Identity;

            return frame;
        }
        /// <summary>
        /// Create a new mesh container for the tank hierarchy
        /// </summary>
        public override MeshContainer CreateMeshContainer(string name, MeshData meshData, 
            ExtendedMaterial[] materials, EffectInstance[] effectInstances, 
            GraphicsStream adjacency, SkinInformation skinInfo)
        {
            TankMeshContainer mesh = new TankMeshContainer();

            // store the variables we care about
            mesh.Name = name;
            mesh.MeshData = meshData;

            return mesh;
        }

        #endregion
    }
}