//
// Shader for using a hemisphere lighting model for the tankers game
//

// Light direction of the sky (it will point straight down
float3 lightDirection = { 0.0f, -1.0f, 0.0f };            

// light color of the sky and ground
float4 skyColor = { 1.0f, 1.0f, 1.0f, 1.0f };
float4 groundColor = { 0.1f, 0.1f, 0.1f, 1.0f };

// Current world, view and projection transforms
float4x3 WorldViewTransform  : WORLDVIEW;
float4x4 ProjectionTransform : PROJECTION;

struct VS_OUTPUT
{
    float4 FinalPosition  : POSITION;
    float4 FinalColor : COLOR0;
};

VS_OUTPUT Main(
    float3 Pos  : POSITION, 
    float3 Norm : NORMAL, 
    float2 TexCoords : TEXCOORD0,
    float  Occ  : TEXCOORD1)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    // Create the Up axis for the hemisphere light
    float3 hemisphereUpAxis = -lightDirection;

    // Calculate the vertex normal
    float3 vertexNormal = normalize(mul(Norm, (float3x3)WorldViewTransform));
    // Now calculate the final hemisphere color
    Out.FinalColor = lerp(skyColor, groundColor, (dot(vertexNormal, hemisphereUpAxis) + 1) / 2) * (1 - Occ);
    // Finally calculate the position after projection
    float3 position = mul(float4(Pos, 1), (float4x3)WorldViewTransform);    // position (view space)
    Out.FinalPosition = mul(float4(position, 1), ProjectionTransform);

    return Out;
}
