using System;
using System.IO;
using System.Collections;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Demo
{
    /// <summary>
    /// The texture pool class
    /// </summary>
    class TexturePool
    {
        // Constant texture path
        private const string TexturePath = "Textures\\";
        // The hash table of valid textures
        private static Hashtable textureTable = new Hashtable();

        /// <summary>
        /// Create a new texture, or return one from the pool
        /// </summary>
        /// <param name="file">Texture file to create</param>
        /// <returns>A texture</returns>
        public static Texture CreateTexture(Device renderDevice, string file)
        {
			try
			{
				// Make sure a texture was passed in
				if ((file == null) || (file.Length == 0))
				{
					// No texutre
					return null;
				}

				// Before we create a texture, return one from the 
				// pool if it exists
				string lowerCaseFile = file.ToLower();
				if (textureTable.ContainsKey(lowerCaseFile))
				{
					return textureTable[lowerCaseFile] as Texture;
				}

				// We've made it here, this is a new texture, Create the texture and add it to the pool
				Texture t = TextureLoader.FromFile(renderDevice, GameEngine.MediaPath + TexturePath + file);
            
				System.Diagnostics.Debug.Assert(t != null, "Created Texture is null", 
					"Textures being loaded by the pool should never be null");

				// Add it to the pool
				textureTable.Add(lowerCaseFile, t);

				return t;
			}
			catch
			{
				System.Diagnostics.Debug.WriteLine("Could not load texture file "+file.ToString());
				return null;
			}
        }


        /// <summary>
        /// Release all textures that have been created thus far
        /// </summary>
        public static void ReleaseAllTextures()
        {
            foreach(string s in textureTable.Keys)
            {
                // Simply dispose of each texture in the list
                (textureTable[s] as Texture).Dispose();
            }

            // Clear the texture list
            textureTable.Clear();
        }

    }
}