//#define USE_HIGH_RES

//this version has a lighting/material problem with anything but the tank mesh.
using System;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.Samples.DirectX.UtilityToolkit;

namespace Demo
{
    /// <summary>
    /// Will hold the possible  tank colors
    /// </summary>
    public enum ShipColors : byte
    {
        Gray,
        Black,
        Green,
        Sand,
        FirstColor = Gray,
        LastColor = Sand,
    }

    /// <summary>
    /// Will hold the data for the tanks
    /// </summary>
    public class Ship : IDisposable
    {
        #region Constants
		public const float ShipLength = 100f;


    #if (USE_HIGH_RES)
        public const string ShipMeshFile = "baker.X";
    #else
        public const string ShipMeshFile = "baker.X";
    #endif

        #endregion

        #region Instance Data
		private Mesh shipMesh = null;
        private Texture[] shipTextures = null;
		private Material[] shipMaterials = null;
		// Stored rendering device
        private Device device = null;
        // Stored direction
        private Vector3 facingDirection;
		
        // Stored rotation matrix
        private Matrix rotationMatrix;
		private Matrix orientationMatrix;
		private int numberSubsets;

        // State values
        private float rotationAngle = 0.0f; // The current rotation angle of the tank itself
		private float orientationAngle = 0.0f;
        private Vector3 shipPosition; // The current position of the tank
        private float shipRadius = 0.0f; // The radius of the bounding sphere the tank encompasses
        private Vector3 shipCenter; // The center of the tank

        #endregion


		#region Properties
		/// <summary>
		/// The tanks current position
		/// </summary>
		public Vector3 Position
		{
			get { return shipPosition; }
			set { shipPosition = value; }
		}
        

		/// <summary>
		/// The rotation angle of the ship
		/// </summary>
		public float ShipRotation
		{
			get 
			{ 
				return rotationAngle; 
			}
			set 
			{ 
				// Set the new rotation angle
				rotationAngle = value; 
				// Update the rotation matrix
				rotationMatrix = Matrix.RotationY(rotationAngle);
				// Calculate the direction you're facing
				facingDirection = new Vector3(rotationMatrix.M31, rotationMatrix.M32
					, rotationMatrix.M33);
				facingDirection.Normalize();
			}
		}

		public float ShipOrientation
		{
			get
			{
				return orientationAngle;
			}
			set
			{
				orientationAngle = value;
				orientationMatrix = Matrix.RotationZ(orientationAngle);
				//facingDirection = new Vector3(rotationMatrix.M31, rotationMatrix.M32
				//	, rotationMatrix.M33);
				//facingDirection.Normalize();
			}
		}

			/// <summary>
			/// The rotation matrix of the tank
			/// </summary>
		public Matrix ShipRotationMatrix
		{
			get { return rotationMatrix; }
			set { rotationMatrix = value; }
		}

		public Matrix ShipOrientationMatrix
		{
			get { return orientationMatrix; }
		}

			/// <summary>
			/// The direction the tank is currently facing
			/// </summary>
			public Vector3 ShipDirection
		{
			get { return facingDirection; }
			set { facingDirection = value; }
		}
		/// <summary>
		/// Radius of the tank
		/// </summary>
		public float Radius
		{
			get { return shipRadius; }
		}

		/// <summary>
		/// The center of the tank
		/// </summary>
		public Vector3 Center
		{
			get { return shipCenter; }
		}

		/// <summary>
		/// Update the tank color
		/// </summary>


		#endregion

        /// <summary>
        /// Create a new tank object
        /// </summary>
        public Ship(Device renderingDevice)
        {
            // Store the device
            device = renderingDevice;

            // Set up the matrices
            ShipRotation = 0.0f;
			orientationMatrix = Matrix.Identity;
			rotationMatrix = Matrix.Identity;

			// First creat our 'flat' hierarchy mesh
			ExtendedMaterial[] mtrl = null;
			shipMesh = Mesh.FromFile(GameEngine.MediaPath + Ship.ShipMeshFile, MeshFlags.Managed,
				renderingDevice, out mtrl);

			// Store the number of subsets
			if ((mtrl != null) && (mtrl.Length > 0) )
			{
				numberSubsets = mtrl.Length;
			}

			System.Diagnostics.Debug.WriteLine("ship subsets = "+numberSubsets.ToString());
			
			VertexBuffer vb = shipMesh.VertexBuffer;
			GraphicsStream vertexData = vb.Lock(0,0,LockFlags.NoSystemLock);
            shipRadius = Geometry.ComputeBoundingSphere(vertexData,shipMesh.NumberVertices,shipMesh.VertexFormat,out shipCenter);
			//shipCenter += new Vector3(-180f,0f,0f);
			shipRadius *= 0.75f;
			System.Diagnostics.Debug.WriteLine("Radius is : "+shipRadius.ToString());
			vb.Unlock();
			vb.Dispose();

			if (shipTextures == null && mtrl != null)
			{
				shipTextures = new Texture[mtrl.Length];
				shipMaterials = new Material[mtrl.Length];
				bool gotTexture = false;
				for ( int i = 0; i < mtrl.Length; i++)
				{
					shipMaterials[i] = mtrl[i].Material3D;
					shipMaterials[i].Ambient = shipMaterials[i].Diffuse;

					try
					{
						if (mtrl[i].TextureFilename != null)
						{
							shipTextures[i] = TextureLoader.FromFile(renderingDevice,
								GameEngine.MediaPath + mtrl[i].TextureFilename);
							//shipTextures[i] = TextureLoader.FromFile(renderingDevice,
							//	GameEngine.MediaPath + "Viper.BMP");
							System.Diagnostics.Debug.WriteLine("Loaded texture Viper " + mtrl[i].TextureFilename);
							gotTexture = true;

						}
						///else
//						{
//							shipTextures[i] = TextureLoader.FromFile(renderingDevice,
//								GameEngine.MediaPath + "Viper.BMP");
//							System.Diagnostics.Debug.WriteLine("Loaded texture Viper2 ");
//							}
					}
					catch
					{
						System.Diagnostics.Debug.WriteLine("Unable to load texture " + mtrl[i].TextureFilename);
					}

				}
//				if (!gotTexture)
//				{
//
//					for ( int i = 0; i < mtrl.Length; i++)
//					{
//						shipTextures[i] = TextureLoader.FromFile(renderingDevice,
//							GameEngine.MediaPath + "Viper.BMP");
//						System.Diagnostics.Debug.WriteLine("Loaded texture " + "LowResGray.PNG");
//					}
//
//				}
			}


			System.Diagnostics.Debug.WriteLine("Vertex equals " + shipMesh.NumberVertices.ToString());

			//shipTexture = GetShipTexture(device, currentColor);
        }

		public Ship(Device renderingDevice, string MeshFile, Color myColor)
		{
			// Store the device
			device = renderingDevice;

			// Set up the matrices
			ShipRotation = 0.0f;
			orientationMatrix = Matrix.Identity;
			rotationMatrix = Matrix.Identity;

			// First creat our 'flat' hierarchy mesh
			ExtendedMaterial[] mtrl = null;
			shipMesh = Mesh.FromFile(GameEngine.MediaPath + MeshFile, MeshFlags.Managed,
				renderingDevice, out mtrl);

			// Store the number of subsets
			if ((mtrl != null) && (mtrl.Length > 0) )
			{
				numberSubsets = mtrl.Length;
			}

			System.Diagnostics.Debug.WriteLine("ship subsets = "+numberSubsets.ToString());
			
			VertexBuffer vb = shipMesh.VertexBuffer;
			GraphicsStream vertexData = vb.Lock(0,0,LockFlags.NoSystemLock);
			shipRadius = Geometry.ComputeBoundingSphere(vertexData,shipMesh.NumberVertices,shipMesh.VertexFormat,out shipCenter);
			if (MeshFile == "viper4.x") shipCenter += new Vector3(-180f,0f,0f);  // viper4 has a bad center
			if (MeshFile == "raider.x") shipCenter += new Vector3(-50f,0f,45f);
			shipRadius *= 0.75f;
			if (shipRadius < 100) shipRadius = 200;
			System.Diagnostics.Debug.WriteLine("Radius is : "+shipRadius.ToString());
			vb.Unlock();
			vb.Dispose();

			if (shipTextures == null && mtrl != null)
			{
				shipTextures = new Texture[mtrl.Length];
				shipMaterials = new Material[mtrl.Length];
				
				for ( int i = 0; i < mtrl.Length; i++)
				{
					shipMaterials[i] = mtrl[i].Material3D;
					
					shipMaterials[i].Ambient = shipMaterials[i].Diffuse = myColor;

					try
					{
						if (mtrl[i].TextureFilename != null)
						{
							shipTextures[i] = TextureLoader.FromFile(renderingDevice,
								GameEngine.MediaPath + mtrl[i].TextureFilename);
							//shipTextures[i] = TextureLoader.FromFile(renderingDevice,
							//	GameEngine.MediaPath + "Viper.BMP");
							System.Diagnostics.Debug.WriteLine("Loaded texture " + mtrl[i].TextureFilename);
							

						}
						///else
						//						{
						//							shipTextures[i] = TextureLoader.FromFile(renderingDevice,
						//								GameEngine.MediaPath + "Viper.BMP");
						//							System.Diagnostics.Debug.WriteLine("Loaded texture Viper2 ");
						//							}
					}
					catch
					{
						System.Diagnostics.Debug.WriteLine("Unable to load texture " + mtrl[i].TextureFilename);
					}

				}
				//				if (!gotTexture)
				//				{
				//
				//					for ( int i = 0; i < mtrl.Length; i++)
				//					{
				//						shipTextures[i] = TextureLoader.FromFile(renderingDevice,
				//							GameEngine.MediaPath + "Viper.BMP");
				//						System.Diagnostics.Debug.WriteLine("Loaded texture " + "LowResGray.PNG");
				//					}
				//
				//				}
			}


			System.Diagnostics.Debug.WriteLine("Vertex equals " + shipMesh.NumberVertices.ToString());

			//shipTexture = GetShipTexture(device, currentColor);
		}

		public Ship(Device renderingDevice, string MeshFile)
		{
			// Store the device
			device = renderingDevice;
			

			// Set up the matrices
			ShipRotation = 0.0f;
			orientationMatrix = Matrix.Identity;
			rotationMatrix = Matrix.Identity;

			// First creat our 'flat' hierarchy mesh
			ExtendedMaterial[] mtrl = null;
			shipMesh = Mesh.FromFile(GameEngine.MediaPath + MeshFile, MeshFlags.Managed,
				renderingDevice, out mtrl);

			// Store the number of subsets
			if ((mtrl != null) && (mtrl.Length > 0) )
			{
				numberSubsets = mtrl.Length;
			}

			System.Diagnostics.Debug.WriteLine("ship subsets = "+numberSubsets.ToString());
			
			VertexBuffer vb = shipMesh.VertexBuffer;
			GraphicsStream vertexData = vb.Lock(0,0,LockFlags.NoSystemLock);
			shipRadius = Geometry.ComputeBoundingSphere(vertexData,shipMesh.NumberVertices,shipMesh.VertexFormat,out shipCenter);
			if (MeshFile == "viper4.x") shipCenter += new Vector3(-180f,0f,0f);  // viper4 has a bad center
			if (MeshFile == "raider.x") shipCenter += new Vector3(-250f,0f,100f);
			shipRadius *= 0.75f;
			if (shipRadius < 100) shipRadius = 200;
			System.Diagnostics.Debug.WriteLine("Radius is : "+shipRadius.ToString());
			vb.Unlock();
			vb.Dispose();

			if (shipTextures == null && mtrl != null)
			{
				shipTextures = new Texture[mtrl.Length];
				shipMaterials = new Material[mtrl.Length];
				bool gotTexture = false;
				for ( int i = 0; i < mtrl.Length; i++)
				{
					shipMaterials[i] = mtrl[i].Material3D;
					
					shipMaterials[i].Ambient = shipMaterials[i].Diffuse;

					try
					{
						if (mtrl[i].TextureFilename != null)
						{
							shipTextures[i] = TextureLoader.FromFile(renderingDevice,
								GameEngine.MediaPath + mtrl[i].TextureFilename);
							//shipTextures[i] = TextureLoader.FromFile(renderingDevice,
							//	GameEngine.MediaPath + "Viper.BMP");
							System.Diagnostics.Debug.WriteLine("Loaded texture " + mtrl[i].TextureFilename);
							gotTexture = true;

						}
						///else
						//						{
						//							shipTextures[i] = TextureLoader.FromFile(renderingDevice,
						//								GameEngine.MediaPath + "Viper.BMP");
						//							System.Diagnostics.Debug.WriteLine("Loaded texture Viper2 ");
						//							}
					}
					catch
					{
						System.Diagnostics.Debug.WriteLine("Unable to load texture " + mtrl[i].TextureFilename);
					}

				}
				//				if (!gotTexture)
				//				{
				//
				//					for ( int i = 0; i < mtrl.Length; i++)
				//					{
				//						shipTextures[i] = TextureLoader.FromFile(renderingDevice,
				//							GameEngine.MediaPath + "Viper.BMP");
				//						System.Diagnostics.Debug.WriteLine("Loaded texture " + "LowResGray.PNG");
				//					}
				//
				//				}
			}


			System.Diagnostics.Debug.WriteLine("Vertex equals " + shipMesh.NumberVertices.ToString());

			//shipTexture = GetShipTexture(device, currentColor);
		}
        /// <summary>
        /// Render the tank, either using the 
        /// </summary>
        /// <param name="renderingDevice"></param>
        public void Draw(GameEngine engine)
        {
			#if(DEBUG)
				//System.Diagnostics.Debug.WriteLine("add me");
				engine.numberFaces += shipMesh.NumberFaces;
				engine.numberVerts += shipMesh.NumberVertices;
			#endif

			//device.RenderState.Lighting = true;

			for (int i=0; i < shipMaterials.Length; i++)
			{
				engine.RenderingDevice.SetTexture(i, shipTextures[i]);
				device.Material = shipMaterials[i];
				engine.RenderingDevice.Material = shipMaterials[i];
				device.SetTexture(i, shipTextures[i]);
				
				//System.Diagnostics.Debug.WriteLine("draw me");
				shipMesh.DrawSubset(i);				
			}
			//device.RenderState.Lighting = true;
        }
        /// <summary>
        /// Update the tank
        /// </summary>
        public void Update(float time)
        {
            // Update the matrices for this tank based on where the tank currently is in the world            
        }
    

        #region IDisposable Members

        public void Dispose()
        {
			if (shipMesh != null)
			{
				shipMesh.Dispose();
			}
        }

        #endregion
    }
  
}