using System;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

namespace Demo
{
    /// <summary>
    /// This class will maintain the sound engine for this game
    /// </summary>
    public class SoundEngine : IDisposable
    {
        #region Constants
        private const string FireFile = @"sounds\fire.wav";
        private const string ExplosionFile = @"sounds\hit.wav";
        #endregion

        #region Instance Data
        private Device soundDevice = null;
        private SecondaryBuffer fireBuffer = null;
        private SecondaryBuffer hitBuffer = null;
        private bool canUseSounds = true;
        #endregion

        #region Creation
        /// <summary>
        /// Constructor where the device will be created.
        /// </summary>
        public SoundEngine(IntPtr parent)
        {
            try
            {
                // Create a default sound object
                soundDevice = new Device();
                // Set the coop level
                soundDevice.SetCooperativeLevel(parent, CooperativeLevel.Normal);

                // Now create the two buffers
                fireBuffer = new SecondaryBuffer(GameEngine.MediaPath + FireFile, soundDevice);
                hitBuffer = new SecondaryBuffer(GameEngine.MediaPath + ExplosionFile, soundDevice);
            }
            catch
            {
                // We cannot use sounds for some reason
                canUseSounds = false;
            }
        }
        #endregion

        #region Play Sound Methods
        /// <summary>
        /// Will play the fire weapon sound
        /// </summary>
        public void FireWeapon()
        {
            if ((canUseSounds) && (fireBuffer != null))
            {
                fireBuffer.Play(0, BufferPlayFlags.Default);
            }
        }
        /// <summary>
        /// Will play the explosion sound
        /// </summary>
        public void Explosion()
        {
            if ((canUseSounds) && (hitBuffer != null))
            {
                hitBuffer.Play(0, BufferPlayFlags.Default);
            }
        }
        #endregion

        #region IDisposable Members

        /// <summary>
        /// Done with this object, clean it up
        /// </summary>
        public void Dispose()
        {
            // Cleanup all data here
            if (fireBuffer != null)
            {
                fireBuffer.Dispose();
            }
            if (hitBuffer != null)
            {
                hitBuffer.Dispose();
            }
            if (soundDevice != null)
            {
                soundDevice.Dispose();
            }
        }

        #endregion
    }
}